# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

import json
from unittest import TestCase

try:
    from urlparse import urlparse, parse_qs     # Python 2
except ImportError:
    from urllib.parse import urlparse, parse_qs # Python 3

from .support.unit import HTTPHelpers
from yampy import Authenticator
from yampy.errors import *


class AuthenticatorAuthorizationUrlTest(TestCase):
    def test_authorization_url_using_the_default_oauth_dialog_url(self):
        authenticator = Authenticator(
            client_id="foo",
            client_secret="bar",
        )
        auth_url_parts, params = self.parsed_authorization_url(authenticator)

        self.assertEqual("https", auth_url_parts.scheme)
        self.assertEqual("www.yammer.com", auth_url_parts.netloc)
        self.assertEqual("/dialog/oauth", auth_url_parts.path)
        self.assertEqual(
            {
                "client_id": ["foo"],
                "redirect_uri": ["http://example.com/auth/callback"],
            },
            params,
        )

    def test_authorization_url_can_use_a_custom_oauth_dialog_url(self):
        authenticator = Authenticator(
            client_id="my-client-id",
            client_secret="bar",
            oauth_dialog_url="http://example.com/test/dialog",
        )
        auth_url_parts, params = self.parsed_authorization_url(authenticator)

        self.assertEqual("http", auth_url_parts.scheme)
        self.assertEqual("example.com", auth_url_parts.netloc)
        self.assertEqual("/test/dialog", auth_url_parts.path)
        self.assertEqual(
            {
                "client_id": ["my-client-id"],
                "redirect_uri": ["http://example.com/auth/callback"],
            },
            params,
        )

    def parsed_authorization_url(self, authenticator):
        auth_url = authenticator.authorization_url(
            redirect_uri="http://example.com/auth/callback",
        )
        auth_url_parts = urlparse(auth_url)
        params = parse_qs(auth_url_parts.query)

        return (auth_url_parts, params)


class AuthenticatorFetchAccessDataTest(HTTPHelpers, TestCase):
    def test_fetch_access_data_returns_parsed_response_json(self):
        self.stub_get_requests(response_body=self.valid_response_json)
        authenticator = Authenticator(client_id="foo", client_secret="bar")

        response_data = authenticator.fetch_access_data("my-code")

        self.assert_get_request(
            url="https://www.yammer.com/oauth2/access_token.json",
            headers={},
            params={
                "client_id": "foo",
                "client_secret": "bar",
                "code": "my-code",
            },
        )
        self.assertEqual(self.valid_response_data, response_data)

    def test_fetch_access_data_can_use_a_custom_oauth_url(self):
        self.stub_get_requests(response_body=self.valid_response_json)
        authenticator = Authenticator(
            client_id="foo",
            client_secret="bar",
            oauth_base_url="http://example.com/oauth",
        )

        authenticator.fetch_access_data("some-code")

        self.assert_get_request("http://example.com/oauth/access_token.json")

    @property
    def valid_response_data(self):
        return {
            "user": {
                "full_name": "Joe Bloggs",
                "state": "active",
            },
            "network": {
                "name": "Acme Inc.",
                "paid": True,
            },
            "access_token": {
                "token": "abc123",
                "view_messages": True,
            },
        }

    @property
    def valid_response_json(self):
        return json.dumps(self.valid_response_data)


class AuthenticatorFetchAccessTokenTest(HTTPHelpers, TestCase):
    def test_fetch_access_token_returns_the_access_token(self):
        self.stub_get_requests(response_body=self.valid_response_json)
        authenticator = Authenticator(client_id="foo", client_secret="bar")

        access_token = authenticator.fetch_access_token("my-code")

        self.assert_get_request(
            url="https://www.yammer.com/oauth2/access_token.json",
            headers={},
            params={
                "client_id": "foo",
                "client_secret": "bar",
                "code": "my-code",
            },
        )
        self.assertEqual("abc123", access_token)

    def test_fetch_access_token_handles_an_unexpected_response_format(self):
        self.stub_get_requests(response_body="""{"foo": "bar"}""")
        authenticator = Authenticator(client_id="foo", client_secret="bar")

        self.assertRaises(ResponseError,
                          authenticator.fetch_access_token, "my-code")

    @property
    def valid_response_json(self):
        return """{
            "user": {
                "full_name": "Joe Bloggs",
                "state": "active"
            },
            "network": {
                "name": "Acme Inc.",
                "paid": true
            },
            "access_token": {
                "token": "abc123",
                "view_messages": true
            }
        }"""



from mock import Mock

from tests.support.unit import TestCaseWithMockClient
from yampy.apis import GroupsAPI


class GroupsAPIAllTest(TestCaseWithMockClient):

    def setUp(self):
        super(GroupsAPIAllTest, self).setUp()
        self.groups_api = GroupsAPI(client=self.mock_client)

    def test_all(self):
        groups = self.groups_api.all()

        self.mock_client.get.assert_called_once_with("/groups")
        self.assertEquals(self.mock_get_response, groups)

    def test_all_with_reverse(self):
        groups = self.groups_api.all(reverse=True)

        self.mock_client.get.assert_called_once_with(
            "/groups",
            reverse="true",
        )
        self.assertEquals(self.mock_get_response, groups)

    def test_find_current(self):
        current_user_groups = self.groups_api.all(mine=True)

        self.mock_client.get.assert_called_once_with("/groups", mine="true")
        self.assertEquals(self.mock_get_response, current_user_groups)


class GroupsAPIFindTest(TestCaseWithMockClient):

    def setUp(self):
        super(GroupsAPIFindTest, self).setUp()
        self.groups_api = GroupsAPI(client=self.mock_client)

    def test_find(self):
        found_group = self.groups_api.find(13)

        self.mock_client.get.assert_called_once_with("/groups/13")
        self.assertEquals(self.mock_get_response, found_group)

    def test_find_passing_id_as_a_dict(self):
        found_group = self.groups_api.find({"id": 31})

        self.mock_client.get.assert_called_once_with("/groups/31")
        self.assertEquals(self.mock_get_response, found_group)

    def test_find_passing_id_as_an_object(self):
        found_group = self.groups_api.find(Mock(id=27))

        self.mock_client.get.assert_called_once_with("/groups/27")
        self.assertEquals(self.mock_get_response, found_group)


class GroupsUsersAPICreateTest(TestCaseWithMockClient):

    def setUp(self):
        super(GroupsUsersAPICreateTest, self).setUp()
        self.groups_api = GroupsAPI(client=self.mock_client)

    def test_create_public_group(self):
        created_group = self.groups_api.create("Test Public group")

        self.mock_client.post.assert_called_once_with(
            "/groups",
            name="Test Public group",
            private="false",
        )
        self.assertEquals(self.mock_post_response, created_group)

    def test_create_private_group(self):
        created_group = self.groups_api.create(
            "Test Private group", private=True)

        self.mock_client.post.assert_called_once_with(
            "/groups",
            name="Test Private group",
            private="true",
        )
        self.assertEquals(self.mock_post_response, created_group)


class GroupsAPIDeleteTest(TestCaseWithMockClient):

    def setUp(self):
        super(GroupsAPIDeleteTest, self).setUp()
        self.groups_api = GroupsAPI(client=self.mock_client)

    def test_delete(self):
        delete_result = self.groups_api.delete(123)

        self.mock_client.delete.assert_called_once_with(
            "/groups/123",
            delete="true",
        )
        self.assertEquals(self.mock_delete_response, delete_result)

    def test_delete_passing_id_as_a_dict(self):
        delete_result = self.groups_api.delete({"id": 171})

        self.mock_client.delete.assert_called_once_with(
            "/groups/171",
            delete="true",
        )
        self.assertEquals(self.mock_delete_response, delete_result)

    def test_delete_passing_id_as_an_object(self):
        delete_result = self.groups_api.delete(Mock(id=125))

        self.mock_client.delete.assert_called_once_with(
            "/groups/125",
            delete="true",
        )
        self.assertEquals(self.mock_delete_response, delete_result)


class GroupsAPIJoinTest(TestCaseWithMockClient):

    def setUp(self):
        super(GroupsAPIJoinTest, self).setUp()
        self.groups_api = GroupsAPI(client=self.mock_client)

    def test_join(self):
        response = self.groups_api.join(171)

        self.mock_client.post.assert_called_once_with(
            "/group_memberships",
            group_id=171,
        )
        self.assertEquals(self.mock_post_response, response)

    def test_join_passing_id_as_a_dict(self):
        response = self.groups_api.join({"id": 171})

        self.mock_client.post.assert_called_once_with(
            "/group_memberships",
            group_id=171,
        )
        self.assertEquals(self.mock_post_response, response)

    def test_join_passing_id_as_an_object(self):
        response = self.groups_api.join(Mock(id=125))

        self.mock_client.post.assert_called_once_with(
            "/group_memberships",
            group_id=125,
        )
        self.assertEquals(self.mock_post_response, response)


class GroupsAPILeaveTest(TestCaseWithMockClient):

    def setUp(self):
        super(GroupsAPILeaveTest, self).setUp()
        self.groups_api = GroupsAPI(client=self.mock_client)

    def test_leave(self):
        self.groups_api.leave(142)

        self.mock_client.delete.assert_called_once_with(
            "/group_memberships",
            group_id=142,
        )

    def test_leave_passing_id_as_a_dict(self):
        self.groups_api.leave({"id": 171})

        self.mock_client.delete.assert_called_once_with(
            "/group_memberships",
            group_id=171,
        )

    def test_leave_passing_id_as_an_object(self):
        self.groups_api.leave(Mock(id=125))

        self.mock_client.delete.assert_called_once_with(
            "/group_memberships",
            group_id=125,
        )



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

"""
Shared support for integration tests.
"""

import json
import logging
from multiprocessing import Process
import requests
from requests.exceptions import RequestException
from unittest import TestCase

import flask


class FakeYammerServer(object):
    """
    A Flask-based web server that simulates the Yammer API by returning
    static JSON data that has a similar structure to the real API's responses.

    The server runs on port 5000.
    """

    def __init__(self):
        """
        Creates the server and defines the various routes.
        """
        self._server = flask.Flask("FakeYammerServer")
        self._silence_logger()

        @self._server.route("/up")
        def up():
            """
            Not part of the Yammer API: This route is polled by the main
            process to determine when the server process is ready.
            """
            return "Server is up"

        @self._server.route("/oauth2/access_token.json")
        def access_token():
            return """{"access_token": {"token": "valid_token"}}"""

        @self._server.route("/api/v1/messages.json", methods=["GET"])
        def messages():
            return self._message_list_json(
                count=3,
                first_id=1,
            )

        @self._server.route("/api/v1/messages.json", methods=["POST"])
        def post_message():
            return self._message_list_json(
                count=1,
                first_id=4,
                body=flask.request.args.get("body"),
            )

        @self._server.route("/api/v1/messages/<id>.json", methods=["DELETE"])
        def delete_message(id):
            return " "

        @self._server.route(
            "/api/v1/messages/liked_by/current.json",
            methods=["POST", "DELETE"],
        )
        def like_or_unlike_message():
            if "message_id" in flask.request.args:
                return " "
            else:
                return " ", 400

        @self._server.route("/api/v1/users.json", methods=["GET"])
        def users():
            return self._user_list_json(count=3, first_id=1)

        @self._server.route("/api/v1/users.json", methods=["POST"])
        def create_user():
            return self._user_json(
                user_id=4,
                first_name=flask.request.args.get("first_name"),
                last_name=flask.request.args.get("last_name"),
            )

        @self._server.route("/api/v1/users/current.json", methods=["GET"])
        def current_user():
            return self._user_json(
                user_id=1,
                first_name="Joe",
                last_name="Bloggs",
            )

        @self._server.route("/api/v1/users/<user_id>.json", methods=["GET"])
        def user(user_id):
            return self._user_json(
                user_id=int(user_id),
                first_name="John",
                last_name="Doe",
            )

        @self._server.route("/api/v1/users/<user_id>.json", methods=["PUT"])
        def update_user(user_id):
            return " "

        @self._server.route("/api/v1/users/<user_id>.json", methods=["DELETE"])
        def delete_user(user_id):
            return " "

    def run_as_process(self):
        """
        Spawns the fake Yammer API server in a new process. Does not return
        until the server is ready to handle requests.
        """
        self._process = Process(target=self._server.run)
        self._process.start()
        self._poll_until_server_responds()

    def stop_process(self):
        """
        Stops the fake Yammer API server's process.
        """
        if hasattr(self, "_process"):
            self._process.terminate()
            self._process.join()

    def _message_list_json(self, count=1, first_id=1, body="Hello world"):
        id_range = xrange(first_id, first_id + count)
        messages = [self._message_dict(msg_id, body) for msg_id in id_range]
        return json.dumps({
            "messages": messages,
            "references": [],
            "meta": [],
        })

    def _message_dict(self, msg_id, body):
        return {
            "id": msg_id,
            "body": {
                "plain": body,
                "parsed": body,
                "rich": body,
            }
        }

    def _user_list_json(self, count=1, first_id=1,
                        first_name="John", last_name="Doe"):
        id_range = xrange(first_id, first_id + count)
        users = [self._user_dict(user_id, first_name, last_name) for user_id in id_range]
        return json.dumps(users)

    def _user_json(self, *args, **kwargs):
        return json.dumps(self._user_dict(*args, **kwargs))

    def _user_dict(self, user_id, first_name, last_name):
        return {
            "id": user_id,
            "first_name": first_name,
            "last_name": last_name,
            "full_name": "%s %s" % (first_name, last_name),
        }

    def _poll_until_server_responds(self):
        try:
            requests.get("http://localhost:5000/up", timeout=0.1)
        except RequestException:
            self._poll_until_server_responds()

    def _silence_logger(self):
        # The info messages from the Flask server are generate by
        # Werkzeug. Increasing its log level silences them.
        logging.getLogger("werkzeug").setLevel(logging.ERROR)


class TestCaseWithFakeYammerServer(TestCase):
    """
    A TestCase that sets up and tears down a FakeYammerServer.

    Subclasses that need to define their own setUp and tearDown methods
    must call the superclass implementations.
    """
    def setUp(self):
        try:
            self._fake_yammer_server = FakeYammerServer()
            self._fake_yammer_server.run_as_process()
        except:
            if hasattr(self, "_fake_yammer_server"):
                self._fake_yammer_server.stop()
            raise

    def tearDown(self):
        self._fake_yammer_server.stop_process()



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

from unittest import TestCase
from mock import patch, Mock

from yampy import Yammer
from yampy.client import Client
from yampy.apis import MessagesAPI, UsersAPI

class YammerTest(TestCase):
    @patch("yampy.yammer.MessagesAPI", spec=True)
    @patch("yampy.yammer.Client", spec=True)
    def test_messages_returns_a_messages_api_instance(self, MockClient,
                                                      MockMessagesAPI):
        yammer = Yammer(access_token="abc123")
        messages = yammer.messages

        MockClient.assert_called_once_with(
            access_token="abc123",
            base_url=None,
            proxies=None,
        )
        MockMessagesAPI.assert_called_once_with(
            client=MockClient(),
        )
        self.assertIsInstance(messages, MessagesAPI)

    @patch("yampy.yammer.UsersAPI", spec=True)
    @patch("yampy.yammer.Client", spec=True)
    def test_users_returns_a_users_api_instance(self, MockClient,
                                                MockUsersAPI):
        yammer = Yammer(access_token="abc123")
        users = yammer.users

        MockClient.assert_called_once_with(
            access_token="abc123",
            base_url=None,
            proxies=None,
        )
        MockUsersAPI.assert_called_once_with(
            client=MockClient(),
        )
        self.assertIsInstance(users, UsersAPI)

    @patch("yampy.yammer.Client", spec=True)
    def test_client_returns_a_client_instance(self, MockClient):
        yammer = Yammer(access_token="thx1138")
        client = yammer.client

        MockClient.assert_called_once_with(
            access_token="thx1138",
            base_url=None,
            proxies=None,
        )
        self.assertIsInstance(client, Client)



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

from mock import Mock

from tests.support.unit import TestCaseWithMockClient
from yampy.apis import MessagesAPI
from yampy.errors import InvalidOpenGraphObjectError, TooManyTopicsError


class MessagesAPIMessageListFetchingTest(TestCaseWithMockClient):
    """
    Tests all MessagesAPI methods associated with fetching lists of messages.
    """

    def setUp(self):
        super(MessagesAPIMessageListFetchingTest, self).setUp()
        self.messages_api = MessagesAPI(client=self.mock_client)

    def test_all(self):
        self._test_list_fetch("/messages", self.messages_api.all)

    def test_from_my_feed(self):
        self._test_list_fetch("/messages/my_feed", self.messages_api.from_my_feed)

    def test_from_top_conversations(self):
        self._test_list_fetch("/messages/algo", self.messages_api.from_top_conversations)

    def test_from_followed_conversations(self):
        self._test_list_fetch("/messages/following", self.messages_api.from_followed_conversations)

    def test_sent(self):
        self._test_list_fetch("/messages/sent", self.messages_api.sent)

    def test_private(self):
        self._test_list_fetch("/messages/private", self.messages_api.private)

    def test_received(self):
        self._test_list_fetch("/messages/received", self.messages_api.received)

    def test_in_thread(self):
        self._test_list_fetch("/messages/in_thread/12345", self.messages_api.in_thread, 12345)

    def test_in_thread_passing_id_as_a_dict(self):
        self._test_list_fetch("/messages/in_thread/321", self.messages_api.in_thread, {"id": 321})

    def test_in_thread_passing_id_as_an_object(self):
        self._test_list_fetch("/messages/in_thread/223", self.messages_api.in_thread, Mock(id=223))

    def test_from_user(self):
        self._test_list_fetch("/messages/from_user/42", self.messages_api.from_user, 42)

    def test_from_user_passing_id_as_a_dict(self):
        self._test_list_fetch("/messages/from_user/31", self.messages_api.from_user, {"id": 31})

    def test_from_user_passing_id_as_an_object(self):
        self._test_list_fetch("/messages/from_user/65", self.messages_api.from_user, Mock(id=65))

    def _test_list_fetch(self, path, method, *method_args):
        for method_kwargs, request_kwargs in self.valid_message_list_arguments:
            messages = method(*method_args, **method_kwargs)

            self.mock_client.get.assert_called_with(path, **request_kwargs)
            self.assertEquals(self.mock_get_response, messages)

    @property
    def valid_message_list_arguments(self):
        """
        Returns tuples, each of which contains the kwargs to pass to a
        MessagesAPI request method, and the kwargs that we expect to be passed
        to the Client.get method as a result.
        """
        return (
            ({},                            {}),
            ({"older_than": 12345},         {"older_than": 12345}),
            ({"older_than": {"id": 7}},     {"older_than": 7}),
            ({"older_than": Mock(id=2)},    {"older_than": 2}),
            ({"newer_than": 54321},         {"newer_than": 54321}),
            ({"newer_than": {"id": 7}},     {"newer_than": 7}),
            ({"newer_than": Mock(id=2)},    {"newer_than": 2}),
            ({"limit": 30},                 {"limit": 30}),
            ({"threaded": True},            {"threaded": "true"}),
            ({"threaded": "extended"},      {"threaded": "extended"}),
        )


class MessagesAPIFindTest(TestCaseWithMockClient):
    def setUp(self):
        super(MessagesAPIFindTest, self).setUp()
        self.messages_api = MessagesAPI(client=self.mock_client)

    def test_find(self):
        message = self.messages_api.find(121)

        self.mock_client.get.assert_called_with("/messages/121")
        self.assertEquals(self.mock_get_response, message)

    def test_find_passing_id_as_a_dict(self):
        self.messages_api.find({"id": 33})

        self.mock_client.get.assert_called_with("/messages/33")

    def test_find_passing_id_as_an_object(self):
        self.messages_api.find(Mock(id=2))

        self.mock_client.get.assert_called_with("/messages/2")


class MessagesAPICreateTest(TestCaseWithMockClient):
    def setUp(self):
        super(MessagesAPICreateTest, self).setUp()
        self.messages_api = MessagesAPI(client=self.mock_client)

    def test_create_simple_message(self):
        response = self.messages_api.create(body="Hello world")

        self.mock_client.post.assert_called_once_with(
            "/messages",
            body="Hello world",
        )
        self.assertEquals(self.mock_post_response, response)

    def test_create_complex_message(self):
        response = self.messages_api.create(
            body="Hi there",
            group_id=123,
            replied_to_id=456,
            direct_to_id=789,
        )

        self.mock_client.post.assert_called_once_with(
            "/messages",
            body="Hi there",
            group_id=123,
            replied_to_id=456,
            direct_to_id=789,
        )

    def test_create_passing_ids_as_dicts(self):
        self.messages_api.create(
            body="Hello world",
            group_id={"id": 123},
            replied_to_id={"id": 456},
            direct_to_id={"id": 789},
        )

        self.mock_client.post.assert_called_once_with(
            "/messages",
            body="Hello world",
            group_id=123,
            replied_to_id=456,
            direct_to_id=789,
        )

    def test_create_passing_ids_as_objects(self):
        self.messages_api.create(
            body="Hello world",
            replied_to_id=Mock(id=37),
        )

        self.mock_client.post.assert_called_once_with(
            "/messages",
            body="Hello world",
            replied_to_id=37,
        )

    def test_create_message_with_topics(self):
        response = self.messages_api.create(
            body="Hi there",
            topics=["unit testing", "yampy"],
        )

        self.mock_client.post.assert_called_once_with(
            "/messages",
            body="Hi there",
            topic1="unit testing",
            topic2="yampy",
        )

    def test_create_message_with_too_many_topics(self):
        with self.assertRaises(TooManyTopicsError):
            self.messages_api.create(
                body="This message has 21 topics",
                topics=["topic %d" % i for i in range(21)],
            )

    def test_create_broadcast_message(self):
        response = self.messages_api.create(
            body="This is a public service announcement",
            broadcast=True,
        )

        self.mock_client.post.assert_called_once_with(
            "/messages",
            body="This is a public service announcement",
            broadcast="true",
        )

    def test_create_message_with_an_open_graph_object(self):
        response = self.messages_api.create(
            body="Google is a search engine",
            open_graph_object={
                "url": "http://www.google.com",
                "fetch": True,
            },
        )

        self.mock_client.post.assert_called_once_with(
            "/messages",
            body="Google is a search engine",
            og_url="http://www.google.com",
            og_fetch="true",
        )

    def test_create_message_with_an_invalid_open_graph_object(self):
        with self.assertRaises(InvalidOpenGraphObjectError):
            self.messages_api.create(
                body="Open graph this!",
                open_graph_object={
                    "fetch": True,
                },
            )


class MessagesAPIDeleteTest(TestCaseWithMockClient):
    def setUp(self):
        super(MessagesAPIDeleteTest, self).setUp()
        self.messages_api = MessagesAPI(client=self.mock_client)

    def test_delete(self):
        response = self.messages_api.delete(3)

        self.mock_client.delete.assert_called_once_with("/messages/3")
        self.assertEquals(self.mock_delete_response, response)

    def test_delete_passing_id_as_a_dict(self):
        self.messages_api.delete({"id": 27})

        self.mock_client.delete.assert_called_once_with("/messages/27")

    def test_delete_passing_id_as_an_object(self):
        self.messages_api.delete(Mock(id=8))

        self.mock_client.delete.assert_called_once_with("/messages/8")


class MessagesAPILikeTest(TestCaseWithMockClient):
    def setUp(self):
        super(MessagesAPILikeTest, self).setUp()
        self.messages_api = MessagesAPI(client=self.mock_client)

    def test_like(self):
        response = self.messages_api.like(42)

        self.mock_client.post.assert_called_once_with(
            "/messages/liked_by/current",
            message_id=42,
        )
        self.assertEquals(self.mock_post_response, response)

    def test_like_passing_id_as_a_dict(self):
        self.messages_api.like({"id": 22})

        self.mock_client.post.assert_called_once_with(
            "/messages/liked_by/current",
            message_id=22,
        )

    def test_like_passing_id_as_an_object(self):
        self.messages_api.like(Mock(id=77))

        self.mock_client.post.assert_called_once_with(
            "/messages/liked_by/current",
            message_id=77,
        )

    def test_unlike(self):
        response = self.messages_api.unlike(42)

        self.mock_client.delete.assert_called_once_with(
            "/messages/liked_by/current",
            message_id=42,
        )
        self.assertEquals(self.mock_delete_response, response)

    def test_unlike_passing_id_as_a_dict(self):
        self.messages_api.unlike({"id": 22})

        self.mock_client.delete.assert_called_once_with(
            "/messages/liked_by/current",
            message_id=22,
        )

    def test_unlike_passing_id_as_an_object(self):
        self.messages_api.unlike(Mock(id=77))

        self.mock_client.delete.assert_called_once_with(
            "/messages/liked_by/current",
            message_id=77,
        )


class MessagesAPIEmailTest(TestCaseWithMockClient):
    def setUp(self):
        super(MessagesAPIEmailTest, self).setUp()
        self.messages_api = MessagesAPI(client=self.mock_client)

    def test_email(self):
        response = self.messages_api.email(13)

        self.mock_client.post.assert_called_once_with(
            "/messages/email",
            message_id=13,
        )
        self.assertEquals(self.mock_post_response, response)

    def test_email_passing_id_as_a_dict(self):
        self.messages_api.email({"id": 4})

        self.mock_client.post.assert_called_once_with(
            "/messages/email",
            message_id=4,
        )

    def test_email_passing_id_as_an_object(self):
        self.messages_api.email(Mock(id=7))

        self.mock_client.post.assert_called_once_with(
            "/messages/email",
            message_id=7,
        )



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

from mock import Mock
from unittest import TestCase

from yampy.apis.utils import ArgumentConverter, IDExtractor, flatten_lists, \
                             flatten_dicts, stringify_booleans, none_filter


class ArgumentConverterNoConvertersTest(TestCase):
    """
    Tests the ArgumentConverter with no converters.
    """

    def setUp(self):
        self.converter = ArgumentConverter()

    def test_does_not_change_any_values(self):
        result = self.converter(
            boolean=True,
            number=1,
            string="hello",
            list=[1, 2, 3],
            tuple=(True, False),
            empty_tuple=(),
            none=None,
        )

        self.assertEquals(True, result["boolean"])
        self.assertEquals(1, result["number"])
        self.assertEquals("hello", result["string"])
        self.assertEquals([1, 2, 3], result["list"])
        self.assertEquals((True, False), result["tuple"])
        self.assertEquals((), result["empty_tuple"])
        self.assertEquals(None, result["none"])


class ArgumentConverterFlattenDictsTest(TestCase):
    """
    Tests the ArgumentConverter using the flatten_dicts converter.
    """

    def setUp(self):
        self.converter = ArgumentConverter(flatten_dicts)

    def test_converts_dict_to_multiple_keys(self):
        result = self.converter(og={"url": "http://www.google.com", "fetch": True})

        with self.assertRaises(KeyError):
            result["og"]

        self.assertEquals("http://www.google.com", result["og_url"])
        self.assertEquals(True, result["og_fetch"])


class ArgumentConverterFlattenListsTest(TestCase):
    """
    Tests the ArgumentConverter using the flatten_lists converter.
    """

    def setUp(self):
        self.converter = ArgumentConverter(flatten_lists)

    def test_converts_list_to_multiple_keys(self):
        result = self.converter(
            topic=["testing", "python"],
        )

        with self.assertRaises(KeyError):
            result["topic"]

        self.assertEquals("testing", result["topic1"])
        self.assertEquals("python", result["topic2"])

    def test_converts_tuple_to_multiple_keys(self):
        result = self.converter(topic=("first", "second", ))

        with self.assertRaises(KeyError):
            result["topic"]

        self.assertEquals("first", result["topic1"])
        self.assertEquals("second", result["topic2"])

    def test_ignores_empty_tuples(self):
        result = self.converter(topic=())

        with self.assertRaises(KeyError):
            result["topic"]

        with self.assertRaises(KeyError):
            result["topic1"]


class ArgumentConverterStringifyBooleansTest(TestCase):
    """
    Tests the ArgumentConverter using the stringify_booleans converter.
    """

    def setUp(self):
        self.converter = ArgumentConverter(stringify_booleans)

    def test_converts_booleans_to_strings(self):
        result = self.converter(
            yes=True,
            no=False,
        )

        self.assertEquals("true", result["yes"])
        self.assertEquals("false", result["no"])


class ArgumentConverterExtractIDsTest(TestCase):
    """
    Tests the ArgumentConverter using the IDExtractor converter.
    """

    def setUp(self):
        self.converter = ArgumentConverter(IDExtractor())

    def test_extracts_ids_from_objects(self):
        result = self.converter(object_id=Mock(id=17))

        self.assertEquals({"object_id": 17}, result)

    def test_extracts_ids_from_dicts(self):
        result = self.converter(dict_id={"id": 31})

        self.assertEquals({"dict_id": 31}, result)

    def test_accepts_primitive_ids(self):
        result = self.converter(
            string_id="123",
            numeric_id=456,
        )

        self.assertEquals("123", result["string_id"])
        self.assertEquals(456, result["numeric_id"])

    def test_ignores_keys_without_an_id_suffix(self):
        id_object = Mock(id=121)
        result = self.converter(
            an_object=id_object,
            a_dict={"id": 45},
        )

        self.assertEquals(id_object, result["an_object"])
        self.assertEquals({"id": 45}, result["a_dict"])

    def test_using_a_custom_regular_expression(self):
        converter = ArgumentConverter(IDExtractor(r'^(older|newer)_than$'))
        result = converter(
            older_than={"id": 1},
            newer_than={"id": 2},
            older_or_newer_than={"id": 3},
            message_id={"id": 4},
        )

        self.assertEquals(1, result["older_than"])
        self.assertEquals(2, result["newer_than"])
        self.assertEquals({"id": 3}, result["older_or_newer_than"])
        self.assertEquals({"id": 4}, result["message_id"])


class ArgumentConverterNoneFilterTest(TestCase):
    """
    Tests the ArgumentConverter using the none_filter converter.
    """

    def setUp(self):
        self.converter = ArgumentConverter(none_filter)

    def test_ignores_none_values(self):
        result = self.converter(foo=None)

        with self.assertRaises(KeyError):
            result["foo"]


class ArgumentConverterCompositionTest(TestCase):
    """
    Tests the ArgumentConverter using multiple converters.
    """

    def setUp(self):
        self.converter = ArgumentConverter(
            flatten_dicts,
            flatten_lists,
            none_filter,
        )

    def test_applies_each_converter_in_turn(self):
        result = self.converter(
            a_dict={
                "a_list": [1, 2, None],
            },
        )

        self.assertEquals(
            {
                "a_dict_a_list1": 1,
                "a_dict_a_list2": 2,
            },
            result
        )



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

from unittest import TestCase

from yampy.models import GenericModel, extract_id


class GenericModelTest(TestCase):
    def test_construction_from_json(self):
        model = GenericModel.from_json('{"foo": 1, "bar": {"sub": 2}}')

        self.assertEquals(1, model.foo)
        self.assertEquals(2, model.bar.sub)

    def test_initialization_with_a_dict(self):
        model = GenericModel({"foo": 1})

        self.assertEquals(1, model.foo)

    def test_missing_attributes(self):
        model = GenericModel({"foo": 1})

        with self.assertRaises(AttributeError):
            model.bar


class ExtractIDTest(TestCase):
    def test_extraction_from_dict(self):
        object_id = extract_id({"id": 17})
        self.assertEquals(17, object_id)

    def test_extraction_from_attribute(self):
        object_id = extract_id(GenericModel({"id": 21}))
        self.assertEquals(21, object_id)

    def test_fallback_when_extraction_fails(self):
        object_id = extract_id(37)
        self.assertEquals(37, object_id)



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

from mock import Mock

from tests.support.unit import TestCaseWithMockClient
from yampy.apis import UsersAPI
from yampy.errors import InvalidEducationRecordError, \
                         InvalidPreviousCompanyRecord


class UsersAPIAllTest(TestCaseWithMockClient):
    def setUp(self):
        super(UsersAPIAllTest, self).setUp()
        self.users_api = UsersAPI(client=self.mock_client)

    def test_all(self):
        users = self.users_api.all()

        self.mock_client.get.assert_called_once_with("/users")
        self.assertEquals(self.mock_get_response, users)

    def test_all_with_pagination(self):
        users = self.users_api.all(page=2)

        self.mock_client.get.assert_called_once_with("/users", page=2)
        self.assertEquals(self.mock_get_response, users)

    def test_all_with_first_letter_of_username(self):
        users = self.users_api.all(letter="g")

        self.mock_client.get.assert_called_once_with("/users", letter="g")
        self.assertEquals(self.mock_get_response, users)

    def test_all_with_custom_sorting(self):
        users = self.users_api.all(sort_by="followers", reverse=True)

        self.mock_client.get.assert_called_once_with(
            "/users",
            sort_by="followers",
            reverse="true",
        )
        self.assertEquals(self.mock_get_response, users)


class UsersAPIInGroupTest(TestCaseWithMockClient):
    def setUp(self):
        super(UsersAPIInGroupTest, self).setUp()
        self.users_api = UsersAPI(client=self.mock_client)

    def test_in_group(self):
        users = self.users_api.in_group(194)

        self.mock_client.get.assert_called_once_with("/users/in_group/194")
        self.assertEquals(self.mock_get_response, users)

    def test_in_group_passing_id_as_a_dict(self):
        self.users_api.in_group({"id": 14})

        self.mock_client.get.assert_called_once_with("/users/in_group/14")

    def test_in_group_passing_id_as_an_object(self):
        self.users_api.in_group(Mock(id=44))

        self.mock_client.get.assert_called_once_with("/users/in_group/44")

    def test_in_group_with_page(self):
        users = self.users_api.in_group(42, page=7)

        self.mock_client.get.assert_called_once_with(
            "/users/in_group/42",
            page=7,
        )
        self.assertEquals(self.mock_get_response, users)


class UsersAPIFindTest(TestCaseWithMockClient):
    def setUp(self):
        super(UsersAPIFindTest, self).setUp()
        self.users_api = UsersAPI(client=self.mock_client)

    def test_find_current(self):
        current_user = self.users_api.find_current()

        self.mock_client.get.assert_called_once_with("/users/current")
        self.assertEquals(self.mock_get_response, current_user)

    def test_find_current_with_additional_information(self):
        current_user = self.users_api.find_current(
            include_followed_users=True,
            include_followed_tags=True,
            include_group_memberships=True,
        )

        self.mock_client.get.assert_called_once_with(
            "/users/current",
            include_followed_users="true",
            include_followed_tags="true",
            include_group_memberships="true",
        )
        self.assertEquals(self.mock_get_response, current_user)

    def test_find(self):
        found_user = self.users_api.find(13)

        self.mock_client.get.assert_called_once_with("/users/13")
        self.assertEquals(self.mock_get_response, found_user)

    def test_find_passing_id_as_a_dict(self):
        self.users_api.find({"id": 31})

        self.mock_client.get.assert_called_once_with("/users/31")

    def test_find_passing_id_as_an_object(self):
        self.users_api.find(Mock(id=27))

        self.mock_client.get.assert_called_once_with("/users/27")

    def test_find_by_email(self):
        found_user = self.users_api.find_by_email("user@example.org")

        self.mock_client.get.assert_called_once_with(
            "/users/by_email",
            email="user@example.org",
        )
        self.assertEquals(self.mock_get_response, found_user)


class UsersAPICreateTest(TestCaseWithMockClient):
    def setUp(self):
        super(UsersAPICreateTest, self).setUp()
        self.users_api = UsersAPI(client=self.mock_client)

    def test_create_simple_user(self):
        created_user = self.users_api.create("user@example.com")

        self.mock_client.post.assert_called_once_with(
            "/users",
            email="user@example.com",
        )
        self.assertEquals(self.mock_post_response, created_user)

    def test_create_complex_user(self):
        created_user = self.users_api.create(
            email_address="someone@example.org",
            full_name="John Doe",
            job_title="Developer",
            location="Stockholm, Sweden",
            im={
                "provider": "gtalk",
                "username": "someone@gmail.example.com",
            },
            work_telephone="+46123123123",
            work_extension="123",
            mobile_telephone="+46789789789",
            significant_other="Jane",
            kids_names="Tom, Dick and Harry",
            interests="Programming, testing",
            summary="Zaphod's just this guy, y'know?",
            expertise="Work and stuff",
        )

        self.mock_client.post.assert_called_once_with(
            "/users",
            email="someone@example.org",
            full_name="John Doe",
            job_title="Developer",
            location="Stockholm, Sweden",
            im_provider="gtalk",
            im_username="someone@gmail.example.com",
            work_telephone="+46123123123",
            work_extension="123",
            mobile_telephone="+46789789789",
            significant_other="Jane",
            kids_names="Tom, Dick and Harry",
            interests="Programming, testing",
            summary="Zaphod's just this guy, y'know?",
            expertise="Work and stuff",
        )

    def test_create_user_with_education_history(self):
        created_user = self.users_api.create(
            email_address="person@example.org",
            education=(
                {
                    "school": "Manchester University",
                    "degree": "BSc",
                    "description": "Computer Science",
                    "start_year": "2002",
                    "end_year": "2005",
                },
                {
                    "school": "Imperial College",
                    "degree": "MSc",
                    "description": "Computer Science",
                    "start_year": "2005",
                    "end_year": "2006",
                }
            ),
        )

        self.mock_client.post.assert_called_once_with(
            "/users",
            email="person@example.org",
            education=[
                "Manchester University,BSc,Computer Science,2002,2005",
                "Imperial College,MSc,Computer Science,2005,2006",
            ]
        )

    def test_create_user_with_invalid_education_history(self):
        with self.assertRaises(InvalidEducationRecordError):
            self.users_api.create(
                email_address="person@example.org",
                education=(
                    {
                        "description": "Computer Science",
                        "start_year": "2005",
                        "end_year": "2006",
                    },
                ),
            )

    def test_create_user_with_a_single_education_record(self):
        self.users_api.create(
            email_address="person@example.org",
            education={
                "school": "MIT",
                "degree": "MSc",
                "description": "Architecture",
                "start_year": "2001",
                "end_year": "2005",
            },
        )

        self.mock_client.post.assert_called_once_with(
            "/users",
            email="person@example.org",
            education=[
                "MIT,MSc,Architecture,2001,2005",
            ]
        )

    def test_create_user_with_employment_history(self):
        created_user = self.users_api.create(
            email_address="ripley@example.org",
            previous_companies=(
                {
                    "company": "Acme Inc.",
                    "position": "developer",
                    "description": "writing code",
                    "start_year": "2001",
                    "end_year": "2012",
                },
                {
                    "company": "Weyland Yutani",
                    "position": "terraforming engineer",
                    "description": "fighting aliens",
                    "start_year": "2110",
                    "end_year": "2119",
                }
            ),
        )

        self.mock_client.post.assert_called_once_with(
            "/users",
            email="ripley@example.org",
            previous_companies=[
                "Acme Inc.,developer,writing code,2001,2012",
                "Weyland Yutani,terraforming engineer,fighting aliens,2110,2119",
            ],
        )

    def test_create_user_with_invalid_employment_history(self):
        with self.assertRaises(InvalidPreviousCompanyRecord):
            self.users_api.create(
                email_address="someone@example.com",
                previous_companies=[
                    {
                        "company": "Incomplete",
                    }
                ],
            )

    def test_create_user_with_a_single_employment_history_record(self):
        self.users_api.create(
            email_address="someone@example.com",
            previous_companies={
                "company": "Blue Sun",
                "position": "T-shirt printer",
                "description": "Making t-shirts",
                "start_year": "2515",
                "end_year": "2517",
            },
        )

        self.mock_client.post.assert_called_once_with(
            "/users",
            email="someone@example.com",
            previous_companies=[
                "Blue Sun,T-shirt printer,Making t-shirts,2515,2517",
            ],
        )


class UsersAPIUpdateTest(TestCaseWithMockClient):
    def setUp(self):
        super(UsersAPIUpdateTest, self).setUp()
        self.users_api = UsersAPI(client=self.mock_client)

    def test_update_user(self):
        result = self.users_api.update(
            user_id=12345,
            full_name="John Doe",
            job_title="Developer",
            location="Stockholm, Sweden",
            im={
                "provider": "gtalk",
                "username": "someone@gmail.example.com",
            },
            work_telephone="+46123123123",
            work_extension="123",
            mobile_telephone="+46789789789",
            significant_other="Jane",
            kids_names="Tom, Dick and Harry",
            interests="Programming, testing",
            summary="Zaphod's just this guy, y'know?",
            expertise="Work and stuff",
        )

        self.mock_client.put.assert_called_once_with(
            "/users/12345",
            full_name="John Doe",
            job_title="Developer",
            location="Stockholm, Sweden",
            im_provider="gtalk",
            im_username="someone@gmail.example.com",
            work_telephone="+46123123123",
            work_extension="123",
            mobile_telephone="+46789789789",
            significant_other="Jane",
            kids_names="Tom, Dick and Harry",
            interests="Programming, testing",
            summary="Zaphod's just this guy, y'know?",
            expertise="Work and stuff",
        )
        self.assertEquals(self.mock_put_response, result)

    def test_update_user_with_education_history(self):
        self.users_api.update(
            user_id=7,
            education=(
                {
                    "school": "Manchester University",
                    "degree": "BSc",
                    "description": "Computer Science",
                    "start_year": "2002",
                    "end_year": "2005",
                },
                {
                    "school": "Imperial College",
                    "degree": "MSc",
                    "description": "Computer Science",
                    "start_year": "2005",
                    "end_year": "2006",
                }
            ),
        )

        self.mock_client.put.assert_called_once_with(
            "/users/7",
            education=[
                "Manchester University,BSc,Computer Science,2002,2005",
                "Imperial College,MSc,Computer Science,2005,2006",
            ]
        )

    def test_update_user_with_invalid_education_history(self):
        with self.assertRaises(InvalidEducationRecordError):
            self.users_api.update(
                user_id=12,
                education=(
                    {
                        "description": "Computer Science",
                        "start_year": "2005",
                        "end_year": "2006",
                    },
                ),
            )

    def test_update_user_with_employment_history(self):
        self.users_api.update(
            user_id=123,
            previous_companies=(
                {
                    "company": "Acme Inc.",
                    "position": "developer",
                    "description": "making software",
                    "start_year": "2001",
                    "end_year": "2012",
                },
                {
                    "company": "Weyland Yutani",
                    "position": "terraforming engineer",
                    "description": "mostly dull routine",
                    "start_year": "2110",
                    "end_year": "2119",
                }
            ),
        )

        self.mock_client.put.assert_called_once_with(
            "/users/123",
            previous_companies=[
                "Acme Inc.,developer,making software,2001,2012",
                "Weyland Yutani,terraforming engineer,mostly dull routine,2110,2119",
            ],
        )

    def test_update_user_with_invalid_employment_history(self):
        with self.assertRaises(InvalidPreviousCompanyRecord):
            self.users_api.update(
                user_id=77,
                previous_companies=[
                    {
                        "company": "Incomplete",
                    }
                ],
            )

    def test_update_user_passing_id_as_a_dict(self):
        self.users_api.update(
            user_id={"id": 11},
            full_name="Joe Bloggs",
        )

        self.mock_client.put.assert_called_once_with(
            "/users/11",
            full_name="Joe Bloggs",
        )

    def test_update_user_passing_id_as_an_object(self):
        self.users_api.update(
            user_id=Mock(id=117),
            full_name="Joe Bloggs",
        )

        self.mock_client.put.assert_called_once_with(
            "/users/117",
            full_name="Joe Bloggs",
        )


class UsersAPISuspendAndDeleteTest(TestCaseWithMockClient):
    def setUp(self):
        super(UsersAPISuspendAndDeleteTest, self).setUp()
        self.users_api = UsersAPI(client=self.mock_client)

    def test_suspend(self):
        suspend_result = self.users_api.suspend(123)

        self.mock_client.delete.assert_called_once_with("/users/123")
        self.assertEquals(self.mock_delete_response, suspend_result)

    def test_suspend_passing_id_as_a_dict(self):
        self.users_api.suspend({"id": 17})

        self.mock_client.delete.assert_called_once_with("/users/17")

    def test_suspend_passing_id_as_an_object(self):
        self.users_api.suspend(Mock(id=12))

        self.mock_client.delete.assert_called_once_with("/users/12")

    def test_delete(self):
        delete_result = self.users_api.delete(123)

        self.mock_client.delete.assert_called_once_with(
            "/users/123",
            delete="true",
        )
        self.assertEquals(self.mock_delete_response, delete_result)



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

from unittest import TestCase

from .support.unit import HTTPHelpers
from yampy import Client
from yampy.errors import *


class ClientGetTest(HTTPHelpers, TestCase):
    def test_get_parses_response_json(self):
        self.stub_get_requests(
            response_body='{"messages": ["first", "second"]}',
        )
        client = Client(access_token="abc123")

        messages = client.get("/messages")

        self.assertEqual(messages.messages, ["first", "second"])

    def test_get_uses_default_base_url(self):
        self.stub_get_requests()
        client = Client(access_token="abc123")

        client.get("/messages")

        self.assert_get_request("https://www.yammer.com/api/v1/messages.json")

    def test_get_uses_custom_base_url(self):
        self.stub_get_requests()
        client = Client(access_token="1a2bc3", base_url="https://example.com")

        client.get("/messages")

        self.assert_get_request("https://example.com/messages.json")

    def test_get_sends_authorization_header(self):
        self.stub_get_requests()
        client = Client(access_token="abc123")

        client.get("/users/123")

        self.assert_get_request(
            url="https://www.yammer.com/api/v1/users/123.json",
            headers={"Authorization": "Bearer abc123"},
        )

    def test_get_does_not_send_authorization_header_with_no_token(self):
        self.stub_get_requests()
        client = Client(access_token=None)

        client.get("/messages")

        self.assert_get_request(
            url="https://www.yammer.com/api/v1/messages.json",
            headers={},
        )

    def test_get_sends_query_string_parameters(self):
        self.stub_get_requests()
        client = Client(access_token="456efg")

        client.get("/users/by_email", email="user@example.com")

        self.assert_get_request(
            url="https://www.yammer.com/api/v1/users/by_email.json",
            params={"email": "user@example.com"},
        )

    def test_get_handles_invalid_access_token_responses(self):
        self.stub_get_requests(
            response_status=400,
            response_body="""{
                "error": {
                    "type": "OAuthException",
                    "message": "Error validating access token."
                 }
             }""",
        )
        client = Client(access_token="456efg")

        self.assertRaises(InvalidAccessTokenError, client.get, "/messages")

    def test_get_handles_unauthorized_responses(self):
        self.stub_get_requests(response_status=401)
        client = Client(access_token="456efg")

        self.assertRaises(UnauthorizedError, client.get, "/messages")

    def test_get_handles_rate_limit_error_responses(self):
        self.stub_get_requests(response_status=429)
        client = Client(access_token="abc")

        self.assertRaises(RateLimitExceededError, client.get, "/user/1")

    def test_get_handles_not_found_responses(self):
        self.stub_get_requests(response_status=404)
        client = Client(access_token="456efg")

        self.assertRaises(NotFoundError, client.get, "/not/real")

    def test_get_handles_unexpected_http_responses(self):
        self.stub_get_requests(response_status=500)
        client = Client(access_token="abcdef")

        self.assertRaises(ResponseError, client.get, "/messages")


class ClientPostTest(HTTPHelpers, TestCase):
    def test_post_parses_response_json(self):
        self.stub_post_requests(
            response_body='{"messages": ["first", "second"]}',
        )
        client = Client(access_token="abc123")

        messages = client.post("/messages", body="Hello world")

        self.assertEqual(messages.messages, ["first", "second"])

    def test_post_uses_default_base_url(self):
        self.stub_post_requests()
        client = Client(access_token="abc123")

        client.post("/messages", body="Hello Yammer")

        self.assert_post_request("https://www.yammer.com/api/v1/messages.json")

    def test_post_uses_custom_base_url(self):
        self.stub_post_requests()
        client = Client(access_token="1a2bc3", base_url="http://example.com")

        client.post("/messages", body="Hello fake Yammer")

        self.assert_post_request("http://example.com/messages.json")

    def test_post_sends_authorization_header(self):
        self.stub_post_requests()
        client = Client(access_token="abc123")

        client.post("/messages", body="I am authorized")

        self.assert_post_request(
            url="https://www.yammer.com/api/v1/messages.json",
            headers={"Authorization": "Bearer abc123"},
        )

    def test_post_sends_query_string_parameters(self):
        self.stub_post_requests()
        client = Client(access_token="456efg")

        client.post("/messages", body="Oh hai")

        self.assert_post_request(
            url="https://www.yammer.com/api/v1/messages.json",
            params={"body": "Oh hai"},
        )

    def test_post_handles_invalid_access_token_responses(self):
        self.stub_post_requests(
            response_status=400,
            response_body="""{
                "error": {
                    "type": "OAuthException",
                    "message": "Error validating access token."
                 }
             }""",
        )
        client = Client(access_token="456efg")

        self.assertRaises(InvalidAccessTokenError, client.post, "/messages",
                          body="No more token")

    def test_post_handles_created_responses(self):
        self.stub_post_requests(
            response_status=201,
            response_body='{"status": "OK"}',
        )
        client = Client(access_token="456efg")

        response = client.post("/messages", body="A-OK")

        self.assertEqual(response, {"status": "OK"})

    def test_post_handles_rate_limit_error_responses(self):
        self.stub_post_requests(response_status=429)
        client = Client(access_token="abc")

        self.assertRaises(RateLimitExceededError, client.post, "/messages",
                          body="Do I talk too much?")

    def test_post_handles_not_found_responses(self):
        self.stub_post_requests(response_status=404)
        client = Client(access_token="456efg")

        self.assertRaises(NotFoundError, client.post, "/not/real")

    def test_post_handles_unexpected_http_responses(self):
        self.stub_post_requests(response_status=122)
        client = Client(access_token="abcdef")

        self.assertRaises(ResponseError, client.post, "/messages",
                          body="BOOM!")


class ClientDeleteTest(HTTPHelpers, TestCase):
    def test_delete_parses_response_json(self):
        self.stub_delete_requests(
            response_body='{"deleted": true}',
        )
        client = Client(access_token="abc123")

        response = client.delete("/messages/1")

        self.assertEqual(response.deleted, True)

    def test_delete_handles_success_with_a_blank_body(self):
        self.stub_delete_requests(response_status=200, response_body=" ")
        client = Client(access_token="foobar")

        response = client.delete("/messages/123")

        self.assertEquals(response, True)

    def test_delete_uses_default_base_url(self):
        self.stub_delete_requests()
        client = Client(access_token="abc123")

        client.delete("/messages/1")

        self.assert_delete_request("https://www.yammer.com/api/v1/messages/1.json")

    def test_delete_uses_custom_base_url(self):
        self.stub_delete_requests()
        client = Client(access_token="1a2bc3", base_url="https://example.com")

        client.delete("/messages/3")

        self.assert_delete_request("https://example.com/messages/3.json")

    def test_delete_sends_authorization_header(self):
        self.stub_delete_requests()
        client = Client(access_token="abc123")

        client.delete("/users/123")

        self.assert_delete_request(
            url="https://www.yammer.com/api/v1/users/123.json",
            headers={"Authorization": "Bearer abc123"},
        )

    def test_delete_sends_query_string_parameters(self):
        self.stub_delete_requests()
        client = Client(access_token="456efg")

        client.delete("/messages/liked_by/current", message_id=12345)

        self.assert_delete_request(
            url="https://www.yammer.com/api/v1/messages/liked_by/current.json",
            params={"message_id": 12345},
        )

    def test_delete_does_not_send_authorization_header_with_no_token(self):
        self.stub_delete_requests()
        client = Client(access_token=None)

        client.delete("/messages/14")

        self.assert_delete_request(
            url="https://www.yammer.com/api/v1/messages/14.json",
            headers={},
        )

    def test_delete_handles_invalid_access_token_responses(self):
        self.stub_delete_requests(
            response_status=400,
            response_body="""{
                "error": {
                    "type": "OAuthException",
                    "message": "Error validating access token."
                 }
             }""",
        )
        client = Client(access_token="456efg")

        self.assertRaises(InvalidAccessTokenError, client.delete, "/messages/1")

    def test_delete_handles_unauthorized_responses(self):
        self.stub_delete_requests(response_status=401)
        client = Client(access_token="456efg")

        self.assertRaises(UnauthorizedError, client.delete, "/messages/1")

    def test_delete_handles_rate_limit_error_responses(self):
        self.stub_delete_requests(response_status=429)
        client = Client(access_token="abc")

        self.assertRaises(RateLimitExceededError, client.delete, "/user/1")

    def test_delete_handles_not_found_responses(self):
        self.stub_delete_requests(response_status=404)
        client = Client(access_token="456efg")

        self.assertRaises(NotFoundError, client.delete, "/not/real")

    def test_delete_handles_unexpected_http_responses(self):
        self.stub_delete_requests(response_status=500)
        client = Client(access_token="abcdef")

        self.assertRaises(ResponseError, client.delete, "/messages/1")


class ClientPutTest(HTTPHelpers, TestCase):
    def test_put_parses_response_json(self):
        self.stub_put_requests(
            response_body='{"messages": ["first", "second"]}',
        )
        client = Client(access_token="abc123")

        messages = client.put("/messages", body="Hello world")

        self.assertEqual(messages.messages, ["first", "second"])

    def test_put_uses_default_base_url(self):
        self.stub_put_requests()
        client = Client(access_token="abc123")

        client.put("/messages", body="Hello Yammer")

        self.assert_put_request("https://www.yammer.com/api/v1/messages.json")

    def test_put_uses_custom_base_url(self):
        self.stub_put_requests()
        client = Client(access_token="1a2bc3", base_url="http://example.com")

        client.put("/messages", body="Hello fake Yammer")

        self.assert_put_request("http://example.com/messages.json")

    def test_put_sends_authorization_header(self):
        self.stub_put_requests()
        client = Client(access_token="abc123")

        client.put("/messages", body="I am authorized")

        self.assert_put_request(
            url="https://www.yammer.com/api/v1/messages.json",
            headers={"Authorization": "Bearer abc123"},
        )

    def test_put_sends_query_string_parameters(self):
        self.stub_put_requests()
        client = Client(access_token="456efg")

        client.put("/messages", body="Oh hai")

        self.assert_put_request(
            url="https://www.yammer.com/api/v1/messages.json",
            params={"body": "Oh hai"},
        )

    def test_put_handles_invalid_access_token_responses(self):
        self.stub_put_requests(
            response_status=400,
            response_body="""{
                "error": {
                    "type": "OAuthException",
                    "message": "Error validating access token."
                 }
             }""",
        )
        client = Client(access_token="456efg")

        self.assertRaises(InvalidAccessTokenError, client.put, "/messages",
                          body="No more token")

    def test_put_handles_created_responses(self):
        self.stub_put_requests(
            response_status=201,
            response_body='{"status": "OK"}',
        )
        client = Client(access_token="456efg")

        response = client.put("/messages", body="A-OK")

        self.assertEqual(response, {"status": "OK"})

    def test_put_handles_rate_limit_error_responses(self):
        self.stub_put_requests(response_status=429)
        client = Client(access_token="abc")

        self.assertRaises(RateLimitExceededError, client.put, "/messages",
                          body="Do I talk too much?")

    def test_put_handles_not_found_responses(self):
        self.stub_put_requests(response_status=404)
        client = Client(access_token="456efg")

        self.assertRaises(NotFoundError, client.put, "/not/real")

    def test_put_handles_unexpected_http_responses(self):
        self.stub_put_requests(response_status=122)
        client = Client(access_token="abcdef")

        self.assertRaises(ResponseError, client.put, "/messages",
                          body="BOOM!")



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

"""
Shared support for unit tests.
"""

from mock import Mock, ANY
import requests
from unittest import TestCase


class HTTPHelpers(object):
    """
    Mixin providing methods for stubbing HTTP requests and asserting that
    the correct tests were made.
    """

    def setUp(self):
        self.__original_request_method = requests.request

    def tearDown(self):
        requests.request = self.__original_request_method

    def stub_get_requests(self, response_body="{}", response_status=200):
        mock_response = Mock(
            text=response_body,
            status_code=response_status,
            reason="",
        )
        requests.request = Mock(return_value=mock_response)

    def assert_get_request(self, url, params=ANY, headers=ANY, proxies=ANY):
        self.assert_request("get", url, params, headers, proxies)

    def stub_post_requests(self, response_body="{}", response_status=200):
        mock_response = Mock(
            text=response_body,
            status_code=response_status,
            reason="",
        )
        requests.request = Mock(return_value=mock_response)

    def assert_post_request(self, url, params=ANY, headers=ANY, proxies=ANY):
        self.assert_request("post", url, params, headers, proxies)

    def stub_delete_requests(self, response_body="{}", response_status=200):
        mock_response = Mock(
            text=response_body,
            status_code=response_status,
            reason="",
        )
        requests.request = Mock(return_value=mock_response)

    def assert_delete_request(self, url, params=ANY, headers=ANY, proxies=ANY):
        self.assert_request("delete", url, params, headers, proxies)

    def stub_put_requests(self, response_body="{}", response_status=200):
        mock_response = Mock(
            text=response_body,
            status_code=response_status,
            reason="",
        )
        requests.request = Mock(return_value=mock_response)

    def assert_put_request(self, url, params=ANY, headers=ANY, proxies=ANY):
        self.assert_request("put", url, params, headers, proxies)

    def assert_request(self, method, url, params=ANY, headers=ANY, proxies=ANY):
        requests.request.assert_called_with(
            method=method,
            url=url,
            params=params,
            headers=headers,
            proxies=proxies,
        )


class TestCaseWithMockClient(TestCase):
    """
    Test case with a mock yampy.Client instance.

    Use self.mock_client in place of a real Client instance. Assert values were
    returned by the mock client by checking against self.mock_get_response, etc.
    """

    def setUp(self):
        self.mock_get_response = Mock()
        self.mock_post_response = Mock()
        self.mock_delete_response = Mock()
        self.mock_put_response = Mock()
        self.mock_client = Mock()
        self.mock_client.get.return_value = self.mock_get_response
        self.mock_client.post.return_value = self.mock_post_response
        self.mock_client.delete.return_value = self.mock_delete_response
        self.mock_client.put.return_value = self.mock_put_response



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

import os

from setuptools import setup
import yampy

VERSION = yampy.__version__

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

setup(
    name="yampy2",
    version=VERSION,
    description="The (un)official Python client for Yammer's API",
    author="Anthony Shaw",
    author_email="anthonyshaw@apache.org",
    long_description=readme + '\n\n' + history,
    packages=["yampy", "yampy.apis"],
    install_requires=["requests"],
    license="Apache License (2.0)",
    url="http://github.com/tonybaloney/yampy2",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'],
    tests_require=["nose", "mock", "flask", "requests"],
    test_suite="nose.collector",
)



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

"""
Tests for excercising the whole system in concert.
"""

from .support.integration import TestCaseWithFakeYammerServer
import yampy


class AuthenticationIntegrationTest(TestCaseWithFakeYammerServer):
    def test_authentication(self):
        authenticator = yampy.Authenticator(
            client_id="valid_client_id",
            client_secret="valid_client_secret",
            oauth_base_url="http://localhost:5000/oauth2",
        )

        access_token = authenticator.fetch_access_token("valid_code")
        self.assertEqual("valid_token", access_token)


class MessageIntegrationTest(TestCaseWithFakeYammerServer):
    def setUp(self):
        super(MessageIntegrationTest, self).setUp()
        self.yammer = yampy.Yammer(
            access_token="valid_token",
            base_url="http://localhost:5000/api/v1",
        )

    def test_fetching_messages(self):
        message_result = self.yammer.messages.all()

        self.assertEqual(3, len(message_result.messages))

    def test_creating_a_message(self):
        message_result = self.yammer.messages.create(
            body="Hello everyone!",
            topics=["python", "testing"],
            group_id=345,
        )

        self.assertEqual(1, len(message_result.messages))
        message = message_result.messages[0]

        self.assertEqual("Hello everyone!", message.body.plain)

    def test_deleting_a_message(self):
        message_result = self.yammer.messages.all()
        msg_id = message_result.messages[0].id

        deletion_result = self.yammer.messages.delete(msg_id)
        self.assertTrue(deletion_result)

    def test_liking_and_unliking_a_message(self):
        message_result = self.yammer.messages.all()
        msg_id = message_result.messages[0].id

        like_result = self.yammer.messages.like(msg_id)
        self.assertTrue(like_result)

        unlike_result = self.yammer.messages.unlike(msg_id)
        self.assertTrue(unlike_result)


class UserIntegrationTest(TestCaseWithFakeYammerServer):
    def setUp(self):
        super(UserIntegrationTest, self).setUp()
        self.yammer = yampy.Yammer(
            access_token="valid_token",
            base_url="http://localhost:5000/api/v1",
        )

    def test_fetching_users(self):
        users = self.yammer.users.all()
        self.assertEqual(3, len(users))

    def test_fetching_individual_users(self):
        current_user = self.yammer.users.find_current()
        self.assertEqual("Joe Bloggs", current_user.full_name)

        user_by_id = self.yammer.users.find(13)
        self.assertEqual(13, user_by_id.id)

    def test_creating_a_user(self):
        user_result = self.yammer.users.create(
            email_address="john.doe@example.com",
            full_name="John Doe",
            education=[
                {
                    "school": "Manchester University",
                    "degree": "BSc",
                    "description": "Computer Science",
                    "start_year": "2002",
                    "end_year": "2005",
                },
            ],
        )

    def test_updating_a_user(self):
        update_result = self.yammer.users.update(
            user_id=1,
            full_name="John Smith",
        )
        self.assertTrue(update_result)

    def test_suspending_a_user(self):
        suspend_result = self.yammer.users.suspend(123)
        self.assertTrue(suspend_result)

    def test_deleting_a_user(self):
        delete_result = self.yammer.users.delete(123)
        self.assertTrue(delete_result)
